@@ -23,6 +23,7 @@
 #include "dm.h"
 #include "ds.h"
 #include "config.h"
+#include "monitor.h"
 #include "notification.h"
 #include "servo.h"
 #include "tlv.h"
@@ -42,6 +43,14 @@ enum clock_type {
 	CLOCK_TYPE_MANAGEMENT = 0x0800,
 };
 
+/**
+ * Appends the active time zone TLVs to a given message.
+ * @param c          The clock instance.
+ * @param m          The message that will receive the TLVs.
+ * @return           Zero on success, non-zero otherwise.
+ */
+int clock_append_timezones(struct clock *c, struct ptp_message *m);
+
 /**
  * Obtains a reference to the best foreign master of a clock.
  * @param c  The clock instance.
@@ -79,6 +88,13 @@ struct config *clock_config(struct clock *c);
  */
 struct currentDS *clock_current_dataset(struct clock *c);
 
+/**
+ * Obtains the clock's data set comparison function.
+ * @param c  The clock instance.
+ * @return   A pointer to the data set comparison function, without fail.
+ */
+int (*clock_dscmp(struct clock *c))(struct dataset *a, struct dataset *b);
+
 /**
  * Obtains the required time stamping mode.
  * @param c  The clock instance.
@@ -134,6 +150,13 @@ struct port *clock_first_port(struct clock *c);
  */
 void clock_follow_up_info(struct clock *c, struct follow_up_info_tlv *f);
 
+/**
+ * Determine if a clock is free running or not.
+ * @param c  The clock instance.
+ * @return   One if the clock is free running or zero otherwise.
+ */
+int clock_free_running(struct clock *c);
+
 /**
  * Obtain the gmCapable flag from a clock's default data set.
  * This function is specific to the 802.1AS standard.
@@ -177,11 +200,10 @@ int clock_manage(struct clock *c, struct port *p, struct ptp_message *msg);
  * Send notification about an event to all subscribers.
  * @param c      The clock instance.
  * @param msg    The PTP message to send, in network byte order.
- * @param msglen The length of the message in bytes.
  * @param event  The event that occured.
  */
 void clock_send_notification(struct clock *c, struct ptp_message *msg,
-			     int msglen, enum notification event);
+			     enum notification event);
 
 /**
  * Construct and send notification to subscribers about an event that
@@ -240,6 +262,27 @@ void clock_set_sde(struct clock *c, int sde);
  */
 int clock_poll(struct clock *c);
 
+/**
+ * Obtain the servo struct.
+ * @param c The clock instance.
+ * @return  A pointer to the clock's servo.
+ */
+struct servo *clock_servo(struct clock *c);
+
+/**
+ * Obtain the current state of clock's servo.
+ * @param c The clock instance.
+ * @return  The current state of the clock's servo.
+ */
+enum servo_state clock_servo_state(struct clock *c);
+
+/**
+ * Obtain the slave monitor instance from a clock.
+ * @param c The clock instance.
+ * @return  The slave monitor associated with the clock.
+ */
+struct monitor *clock_slave_monitor(struct clock *c);
+
 /**
  * Obtain the slave-only flag from a clock's default data set.
  * @param c  The clock instance.
@@ -247,6 +290,20 @@ int clock_poll(struct clock *c);
  */
 int clock_slave_only(struct clock *c);
 
+/**
+ * Obtain the max steps removed field from a clock's default data set.
+ * @param c  The clock instance.
+ * @return   The value of the clock's max steps removed field.
+ */
+UInteger8 clock_max_steps_removed(struct clock *c);
+
+/**
+ * Obtain the clock class threshold field from a clock's default data set.
+ * @param c  The clock instance.
+ * @return   Configured clock class threshold value.
+ */
+UInteger8 clock_get_clock_class_threshold(struct clock *c);
+
 /**
  * Obtain the steps removed field from a clock's current data set.
  * @param c  The clock instance.
@@ -254,6 +311,13 @@ int clock_slave_only(struct clock *c);
  */
 UInteger16 clock_steps_removed(struct clock *c);
 
+/**
+ * Obtain the Time Stamp Processor instance from a clock.
+ * @param c The clock instance.
+ * @return  The Time Stamp Processor associated with the clock.
+ */
+struct tsproc *clock_get_tsproc(struct clock *c);
+
 /**
  * Switch to a new PTP Hardware Clock, for use with the "jbod" mode.
  * @param c          The clock instance.
@@ -283,12 +347,19 @@ enum servo_state clock_synchronize(struct clock *c, tmv_t ingress,
  */
 void clock_sync_interval(struct clock *c, int n);
 
+/**
+ * Update the clock leap bits and UTC offset after a leap second
+ * if operating as a grandmaster.
+ * @param c  The clock instance.
+ */
+void clock_update_leap_status(struct clock *c);
+
 /**
  * Obtain a clock's time properties data set.
  * @param c  The clock instance.
- * @return   A pointer to the time properties data set of the clock.
+ * @return   A copy of the clock's time properties data set.
  */
-struct timePropertiesDS *clock_time_properties(struct clock *c);
+struct timePropertiesDS clock_time_properties(struct clock *c);
 
 /**
  * Update a clock's time properties data set.
