@@ -54,6 +54,7 @@ enum config_type {
 	CFG_TYPE_DOUBLE,
 	CFG_TYPE_ENUM,
 	CFG_TYPE_STRING,
+	CFG_TYPE_UINT,
 };
 
 struct config_enum {
@@ -65,6 +66,7 @@ typedef union {
 	int i;
 	double d;
 	char *s;
+	uint32_t u;
 } any_t;
 
 #define CONFIG_LABEL_SIZE 64
@@ -109,6 +111,14 @@ struct config_item {
 	.min.i	= _min,					\
 	.max.i	= _max,					\
 }
+#define CONFIG_ITEM_UINT(_label, _port, _default, _min, _max) {	\
+	.label	= _label,				\
+	.type	= CFG_TYPE_UINT,			\
+	.flags	= _port ? CFG_ITEM_PORT : 0,		\
+	.val.u	= _default,				\
+	.min.u	= _min,					\
+	.max.u	= _max,					\
+}
 #define CONFIG_ITEM_STRING(_label, _port, _default) {	\
 	.label	= _label,				\
 	.type	= CFG_TYPE_STRING,			\
@@ -125,6 +135,9 @@ struct config_item {
 #define GLOB_ITEM_INT(label, _default, min, max) \
 	CONFIG_ITEM_INT(label, 0, _default, min, max)
 
+#define GLOB_ITEM_UIN(label, _default, min, max) \
+	CONFIG_ITEM_UINT(label, 0, _default, min, max)
+
 #define GLOB_ITEM_STR(label, _default) \
 	CONFIG_ITEM_STRING(label, 0, _default)
 
@@ -137,6 +150,9 @@ struct config_item {
 #define PORT_ITEM_INT(label, _default, min, max) \
 	CONFIG_ITEM_INT(label, 1, _default, min, max)
 
+#define PORT_ITEM_UIN(label, _default, min, max) \
+	CONFIG_ITEM_UINT(label, 1, _default, min, max)
+
 #define PORT_ITEM_STR(label, _default) \
 	CONFIG_ITEM_STRING(label, 1, _default)
 
@@ -171,6 +187,7 @@ static struct config_enum delay_filter_enu[] = {
 
 static struct config_enum delay_mech_enu[] = {
 	{ "Auto", DM_AUTO },
+	{ "COMMON_P2P", DM_COMMON_P2P },
 	{ "E2E",  DM_E2E },
 	{ "P2P",  DM_P2P },
 	{ "NONE", DM_NO_MECHANISM },
@@ -235,6 +252,8 @@ static struct config_enum bmca_enu[] = {
 };
 
 struct config_item config_tab[] = {
+	PORT_ITEM_UIN("active_key_id", 0, 0, UINT32_MAX),
+	PORT_ITEM_INT("allow_unauth", 0, 0, 2),
 	PORT_ITEM_INT("allowedLostResponses", 3, 1, 255),
 	PORT_ITEM_INT("announceReceiptTimeout", 3, 2, UINT8_MAX),
 	PORT_ITEM_ENU("asCapable", AS_CAPABLE_AUTO, as_capable_enu),
@@ -249,6 +268,11 @@ struct config_item config_tab[] = {
 	GLOB_ITEM_INT("clock_class_threshold", CLOCK_CLASS_THRESHOLD_DEFAULT, 6, CLOCK_CLASS_THRESHOLD_DEFAULT),
 	GLOB_ITEM_ENU("clock_servo", CLOCK_SERVO_PI, clock_servo_enu),
 	GLOB_ITEM_ENU("clock_type", CLOCK_TYPE_ORDINARY, clock_type_enu),
+	PORT_ITEM_STR("cmlds.client_address", "/var/run/cmlds_client"),
+	PORT_ITEM_INT("cmlds.domainNumber", 0, 0, 255),
+	PORT_ITEM_INT("cmlds.majorSdoId", 2, 0, 0x0F),
+	PORT_ITEM_INT("cmlds.port", 0, 0, UINT16_MAX),
+	PORT_ITEM_STR("cmlds.server_address", "/var/run/cmlds_server"),
 	GLOB_ITEM_ENU("dataset_comparison", DS_CMP_IEEE1588, dataset_comp_enu),
 	PORT_ITEM_INT("delayAsymmetry", 0, INT_MIN, INT_MAX),
 	PORT_ITEM_ENU("delay_filter", FILTER_MOVING_MEDIAN, delay_filter_enu),
@@ -299,6 +323,9 @@ struct config_item config_tab[] = {
 	GLOB_ITEM_INT("offsetScaledLogVariance", 0xffff, 0, UINT16_MAX),
 	PORT_ITEM_INT("operLogPdelayReqInterval", 0, INT8_MIN, INT8_MAX),
 	PORT_ITEM_INT("operLogSyncInterval", 0, INT8_MIN, INT8_MAX),
+	PORT_ITEM_STR("p2p_dst_ipv4", "224.0.0.107"),
+	PORT_ITEM_STR("p2p_dst_ipv6", "FF02:0:0:0:0:0:0:6B"),
+	PORT_ITEM_STR("p2p_dst_mac", "01:80:C2:00:00:0E"),
 	PORT_ITEM_INT("path_trace_enabled", 0, 0, 1),
 	PORT_ITEM_INT("phc_index", -1, -1, INT_MAX),
 	GLOB_ITEM_DBL("pi_integral_const", 0.0, 0.0, DBL_MAX),
@@ -317,11 +344,13 @@ struct config_item config_tab[] = {
 	GLOB_ITEM_INT("priority1", 128, 0, UINT8_MAX),
 	GLOB_ITEM_INT("priority2", 128, 0, UINT8_MAX),
 	GLOB_ITEM_STR("productDescription", ";;"),
+	PORT_ITEM_STR("ptp_dst_ipv4", "224.0.1.129"),
+	PORT_ITEM_STR("ptp_dst_ipv6", "FF0E:0:0:0:0:0:0:181"),
 	PORT_ITEM_STR("ptp_dst_mac", "01:1B:19:00:00:00"),
-	PORT_ITEM_STR("p2p_dst_mac", "01:80:C2:00:00:0E"),
 	GLOB_ITEM_INT("ptp_minor_version", 1, 0, 1),
 	GLOB_ITEM_STR("refclock_sock_address", "/var/run/refclock.ptp.sock"),
 	GLOB_ITEM_STR("revisionData", ";;"),
+	GLOB_ITEM_STR("sa_file", NULL),
 	GLOB_ITEM_INT("sanity_freq_limit", 200000000, 0, INT_MAX),
 	PORT_ITEM_INT("serverOnly", 0, 0, 1),
 	GLOB_ITEM_INT("servo_num_offset_values", 10, 0, INT_MAX),
@@ -329,6 +358,7 @@ struct config_item config_tab[] = {
 	GLOB_ITEM_STR("slave_event_monitor", ""),
 	GLOB_ITEM_INT("slaveOnly", 0, 0, 1), /*deprecated*/
 	GLOB_ITEM_INT("socket_priority", 0, 0, 15),
+	PORT_ITEM_INT("spp", -1, -1, UINT8_MAX),
 	GLOB_ITEM_DBL("step_threshold", 0.0, 0.0, DBL_MAX),
 	GLOB_ITEM_INT("step_window", 0, 0, INT_MAX),
 	GLOB_ITEM_INT("summary_interval", 0, INT_MIN, INT_MAX),
@@ -340,8 +370,10 @@ struct config_item config_tab[] = {
 	PORT_ITEM_INT("ts2phc.channel", 0, 0, INT_MAX),
 	PORT_ITEM_INT("ts2phc.extts_correction", 0, INT_MIN, INT_MAX),
 	PORT_ITEM_ENU("ts2phc.extts_polarity", PTP_RISING_EDGE, extts_polarity_enu),
+	PORT_ITEM_INT("ts2phc.holdover", 0, 0, INT_MAX),
 	PORT_ITEM_INT("ts2phc.master", 0, 0, 1),
 	PORT_ITEM_INT("ts2phc.nmea_baudrate", 9600, 300, INT_MAX),
+	PORT_ITEM_INT("ts2phc.nmea_delay", 0, INT_MIN, INT_MAX),
 	GLOB_ITEM_STR("ts2phc.nmea_remote_host", ""),
 	GLOB_ITEM_STR("ts2phc.nmea_remote_port", ""),
 	GLOB_ITEM_STR("ts2phc.nmea_serialport", "/dev/ttyS0"),
@@ -563,6 +595,7 @@ static enum parser_result parse_item(struct config *cfg,
 	struct config_enum *cte;
 	double df;
 	int val;
+	uint32_t uval;
 
 	r = parse_fault_interval(cfg, section, option, value);
 	if (r != NOT_PARSED)
@@ -595,6 +628,9 @@ static enum parser_result parse_item(struct config *cfg,
 	case CFG_TYPE_STRING:
 		r = PARSED_OK;
 		break;
+	case CFG_TYPE_UINT:
+		r = get_ranged_uint(value, &uval, cgi->min.u, cgi->max.u);
+		break;
 	}
 	if (r != PARSED_OK) {
 		return r;
@@ -639,6 +675,9 @@ static enum parser_result parse_item(struct config *cfg,
 		}
 		dst->flags |= CFG_ITEM_DYNSTR;
 		break;
+	case CFG_TYPE_UINT:
+		dst->val.u = uval;
+		break;
 	}
 
 	if (commandline) {
@@ -814,8 +853,8 @@ int config_read(const char *name, struct config *cfg)
 
 		if (parse_section_line(line, &current_section) == PARSED_OK) {
 			if (current_section == PORT_SECTION) {
-				char port[17];
-				if (1 != sscanf(line, " %16s", port)) {
+				char port[129];
+				if (1 != sscanf(line, " %128s", port)) {
 					fprintf(stderr, "could not parse port name on line %d\n",
 							line_num);
 					goto parse_error;
@@ -896,7 +935,7 @@ struct interface *config_create_interface(const char *name, struct config *cfg)
 			return iface;
 	}
 
-	iface = interface_create(name);
+	iface = interface_create(name, NULL);
 	if (!iface) {
 		fprintf(stderr, "cannot allocate memory for a port\n");
 		return NULL;
@@ -920,6 +959,7 @@ struct config *config_create(void)
 	}
 	STAILQ_INIT(&cfg->interfaces);
 	STAILQ_INIT(&cfg->unicast_master_tables);
+	STAILQ_INIT(&cfg->security_association_database);
 
 	cfg->opts = config_alloc_longopts();
 	if (!cfg->opts) {
@@ -1017,6 +1057,7 @@ int config_get_int(struct config *cfg, const char *section, const char *option)
 	switch (ci->type) {
 	case CFG_TYPE_DOUBLE:
 	case CFG_TYPE_STRING:
+	case CFG_TYPE_UINT:
 		pr_err("bug: config option %s type mismatch!", option);
 		exit(-1);
 	case CFG_TYPE_INT:
@@ -1027,6 +1068,19 @@ int config_get_int(struct config *cfg, const char *section, const char *option)
 	return ci->val.i;
 }
 
+uint32_t config_get_uint(struct config *cfg, const char *section,
+			 const char *option)
+{
+	struct config_item *ci = config_find_item(cfg, section, option);
+
+	if (!ci || ci->type != CFG_TYPE_UINT) {
+		pr_err("bug: config option %s missing or invalid!", option);
+		exit(-1);
+	}
+	pr_debug("config item %s.%s is %u", section, option, ci->val.u);
+	return ci->val.u;
+}
+
 char *config_get_string(struct config *cfg, const char *section,
 			const char *option)
 {
@@ -1134,6 +1188,7 @@ int config_set_section_int(struct config *cfg, const char *section,
 	switch (cgi->type) {
 	case CFG_TYPE_DOUBLE:
 	case CFG_TYPE_STRING:
+	case CFG_TYPE_UINT:
 		pr_err("bug: config option %s type mismatch!", option);
 		return -1;
 	case CFG_TYPE_INT:
@@ -1159,6 +1214,20 @@ int config_set_section_int(struct config *cfg, const char *section,
 	return 0;
 }
 
+int config_set_uint(struct config *cfg, const char *option, uint32_t val)
+{
+	struct config_item *ci = config_find_item(cfg, NULL, option);
+
+	if (!ci || ci->type != CFG_TYPE_UINT) {
+		pr_err("bug: config option %s missing or invalid!", option);
+		return -1;
+	}
+	ci->flags |= CFG_ITEM_LOCKED;
+	ci->val.u = val;
+	pr_debug("locked item global.%s as %u", option, ci->val.u);
+	return 0;
+}
+
 int config_set_string(struct config *cfg, const char *option,
 		      const char *val)
 {
