@@ -72,14 +72,24 @@ static int check_clock_identity(struct pmc_agent *node, struct ptp_message *msg)
 
 static int is_msg_mgt(struct ptp_message *msg)
 {
+	struct tlv_extra *extra;
 	struct TLV *tlv;
 
 	if (msg_type(msg) != MANAGEMENT)
 		return 0;
 	if (management_action(msg) != RESPONSE)
 		return 0;
-	if (msg_tlv_count(msg) != 1)
+	switch (msg_tlv_count(msg)) {
+	case 1:
+		break;
+	case 2:
+		extra = TAILQ_LAST(&msg->tlv_list, tlv_list);
+		if (extra->tlv->type == TLV_AUTHENTICATION) {
+			break;
+		}
+	default:
 		return 0;
+	}
 	tlv = (struct TLV *) msg->management.suffix;
 	if (tlv->type == TLV_MANAGEMENT)
 		return 1;
@@ -232,9 +242,10 @@ int run_pmc_wait_sync(struct pmc_agent *node, int timeout)
 int init_pmc_node(struct config *cfg, struct pmc_agent *node, const char *uds,
 		  pmc_node_recv_subscribed_t *recv_subscribed, void *context)
 {
-	node->pmc = pmc_create(cfg, TRANS_UDS, uds, 0,
+	node->pmc = pmc_create(cfg, TRANS_UDS, uds,
+			       config_get_string(cfg, NULL, "uds_address"), 0,
 			       config_get_int(cfg, NULL, "domainNumber"),
-			       config_get_int(cfg, NULL, "transportSpecific") << 4, 1);
+			       config_get_int(cfg, NULL, "transportSpecific") << 4, 0, 1);
 	if (!node->pmc) {
 		pr_err("failed to create pmc");
 		return -1;
