@@ -36,6 +36,7 @@
 #include "port_private.h"
 #include "print.h"
 #include "rtnl.h"
+#include "sad.h"
 #include "sk.h"
 #include "tc.h"
 #include "tlv.h"
@@ -46,6 +47,8 @@
 #include "util.h"
 
 #define ANNOUNCE_SPAN 1
+#define CMLDS_SUBSCRIPTION_INTERVAL	60 /*seconds*/
+#define CMLDS_UPDATE_INTERVAL		(CMLDS_SUBSCRIPTION_INTERVAL / 2)
 
 enum syfu_event {
 	SYNC_MISMATCH,
@@ -56,6 +59,7 @@ enum syfu_event {
 
 static int port_is_ieee8021as(struct port *p);
 static int port_is_uds(struct port *p);
+static int port_has_security(struct port *p);
 static void port_nrate_initialize(struct port *p);
 
 static int announce_compare(struct ptp_message *m1, struct ptp_message *m2)
@@ -673,7 +677,13 @@ static int peer_prepare_and_send(struct port *p, struct ptp_message *msg,
 				 enum transport_event event)
 {
 	int cnt;
-	if (msg_pre_send(msg)) {
+	if (port_has_security(p)) {
+		cnt = sad_append_auth_tlv(clock_config(p->clock), p->spp,
+					  p->active_key_id, msg);
+	} else {
+		cnt = msg_pre_send(msg);
+	}
+	if (cnt) {
 		return -1;
 	}
 	if (msg_unicast(msg)) {
@@ -698,6 +708,11 @@ int port_capable(struct port *p)
 		goto capable;
 	}
 
+	if (p->delayMechanism == DM_COMMON_P2P) {
+		/* asCapable is calculated by the CMLDS. */
+		return p->asCapable != NOT_CAPABLE ? 1 : 0;
+	}
+
 	if (tmv_to_nanoseconds(p->peer_delay) >	p->neighborPropDelayThresh) {
 		if (p->asCapable)
 			pr_debug("%s: peer_delay (%" PRId64 ") > neighborPropDelayThresh "
@@ -748,6 +763,7 @@ capable:
 	if (p->asCapable == NOT_CAPABLE) {
 		pr_debug("%s: setting asCapable", p->log_name);
 		p->asCapable = AS_CAPABLE;
+		port_notify_event(p, NOTIFY_CMLDS);
 	}
 	return 1;
 
@@ -755,6 +771,7 @@ not_capable:
 	if (p->asCapable)
 		port_nrate_initialize(p);
 	p->asCapable = NOT_CAPABLE;
+	port_notify_event(p, NOTIFY_CMLDS);
 	return 0;
 }
 
@@ -779,6 +796,15 @@ static int port_ignore(struct port *p, struct ptp_message *m)
 	if (path_trace_ignore(p, m)) {
 		return 1;
 	}
+	if (port_is_ieee8021as(p)) {
+		if (msg_transport_specific(m) != TS_IEEE_8021AS &&
+		    msg_transport_specific(m) != TS_CMLDS) {
+			pr_debug("%s: received %s with invalid transport specific value %d "
+				"for IEEE8021as", p->log_name, msg_type_string(msg_type(m)),
+				msg_transport_specific(m)>>4);
+			return 1;
+		}
+	}
 	if (p->match_transport_specific &&
 	    msg_transport_specific(m) != p->transportSpecific) {
 		return 1;
@@ -858,6 +884,11 @@ static int port_is_uds(struct port *p)
 	return transport_type(p->trp) == TRANS_UDS;
 }
 
+static int port_has_security(struct port *p)
+{
+	return p->spp >= 0;
+}
+
 static void port_management_send_error(struct port *p, struct port *ingress,
 				       struct ptp_message *msg, int error_id)
 {
@@ -883,6 +914,7 @@ static int port_management_fill_response(struct port *target,
 	struct clock_description *desc;
 	struct port_properties_np *ppn;
 	struct port_hwclock_np *phn;
+	struct cmlds_info_np *cmlds;
 	struct management_tlv *tlv;
 	struct port_stats_np *psn;
 	struct foreign_clock *fc;
@@ -962,7 +994,8 @@ static int port_management_fill_response(struct port *target,
 		ptp_text_copy(cd->userDescription, &desc->userDescription);
 		buf += sizeof(struct PTPText) + cd->userDescription->length;
 
-		if (target->delayMechanism == DM_P2P) {
+		if (target->delayMechanism == DM_P2P ||
+		    target->delayMechanism == DM_COMMON_P2P) {
 			memcpy(buf, profile_id_p2p, PROFILE_ID_LEN);
 		} else {
 			struct config *cfg = clock_config(target->clock);
@@ -1125,6 +1158,14 @@ static int port_management_fill_response(struct port *target,
 		memcpy(pwr, &target->pwr, sizeof(*pwr));
 		datalen = sizeof(*pwr);
 		break;
+	case MID_CMLDS_INFO_NP:
+		cmlds = (struct cmlds_info_np *)tlv->data;
+		cmlds->meanLinkDelay = target->peerMeanPathDelay;
+		cmlds->scaledNeighborRateRatio =
+			(Integer32) (target->nrate.ratio * POW2_41 - POW2_41);
+		cmlds->as_capable = target->asCapable;
+		datalen = sizeof(*cmlds);
+		break;
 	default:
 		/* The caller should *not* respond to this message. */
 		tlv_extra_recycle(extra);
@@ -1264,14 +1305,16 @@ int port_set_delay_tmo(struct port *p)
 	if (p->inhibit_delay_req) {
 		return 0;
 	}
-
-	if (p->delayMechanism == DM_P2P) {
+	switch (p->delayMechanism) {
+	case DM_COMMON_P2P:
+	case DM_P2P:
 		return set_tmo_log(p->fda.fd[FD_DELAY_TIMER], 1,
-			       p->logPdelayReqInterval);
-	} else {
-		return set_tmo_random(p->fda.fd[FD_DELAY_TIMER], 0, 2,
-				p->logMinDelayReqInterval);
+				   p->logPdelayReqInterval);
+	default:
+		break;
 	}
+	return set_tmo_random(p->fda.fd[FD_DELAY_TIMER], 0, 2,
+			      p->logMinDelayReqInterval);
 }
 
 static int port_set_manno_tmo(struct port *p)
@@ -1371,6 +1414,8 @@ static void port_synchronize(struct port *p,
 		break;
 	}
 
+	sad_set_last_seqid(clock_config(p->clock), p->spp, seqid);
+
 	last_state = clock_servo_state(p->clock);
 	state = clock_synchronize(p->clock, t2, t1c);
 	switch (state) {
@@ -1517,6 +1562,49 @@ static void port_syfufsm(struct port *p, enum syfu_event event,
 	}
 }
 
+static int port_cmlds_renew(struct port *p, time_t now)
+{
+	struct subscribe_events_np sen = {
+		.duration = CMLDS_SUBSCRIPTION_INTERVAL,
+	};
+	int err;
+
+	event_bitmask_set(sen.bitmask, NOTIFY_CMLDS, TRUE);
+	err = pmc_send_set_action(p->cmlds.pmc, MID_SUBSCRIBE_EVENTS_NP,
+				  &sen, sizeof(sen));
+	if (err) {
+		return err;
+	}
+	p->cmlds.last_renewal = now;
+	return 0;
+}
+
+static enum fsm_event port_cmlds_timeout(struct port *p)
+{
+	struct timespec now;
+	int err;
+
+	if (!p->cmlds.pmc) {
+		return EV_NONE;
+	}
+	clock_gettime(CLOCK_MONOTONIC, &now);
+	if (now.tv_sec - p->cmlds.last_renewal > CMLDS_UPDATE_INTERVAL) {
+		err = port_cmlds_renew(p, now.tv_sec);
+		if (err) {
+			return EV_FAULT_DETECTED;
+		}
+	}
+	p->cmlds.timer_count++;
+	if (p->cmlds.timer_count > p->allowedLostResponses) {
+		p->asCapable = NOT_CAPABLE;
+		err = port_cmlds_renew(p, now.tv_sec);
+		if (err) {
+			return EV_FAULT_DETECTED;
+		}
+	}
+	return EV_NONE;
+}
+
 static int port_pdelay_request(struct port *p)
 {
 	struct ptp_message *msg;
@@ -1860,6 +1948,33 @@ static void port_clear_fda(struct port *p, int count)
 		p->fda.fd[i] = -1;
 }
 
+static int port_cmlds_initialize(struct port *p)
+{
+	struct config *cfg = clock_config(p->clock);
+	const int zero_datalen = 1;
+	const UInteger8 hops = 0;
+	struct timespec now;
+
+	p->cmlds.port = config_get_int(cfg, p->name, "cmlds.port");
+	if (!p->cmlds.port) {
+		p->cmlds.port = portnum(p);
+	}
+	p->cmlds.pmc = pmc_create(cfg, TRANS_UDS,
+				  config_get_string(cfg, p->name, "cmlds.client_address"),
+				  config_get_string(cfg, p->name, "cmlds.server_address"),
+				  hops,
+				  config_get_int(cfg, p->name, "cmlds.domainNumber"),
+				  config_get_int(cfg, p->name, "cmlds.majorSdoId") << 4,
+				  0, zero_datalen);
+	if (!p->cmlds.pmc) {
+		return -1;
+	}
+	p->cmlds.timer_count = 0;
+	p->fda.fd[FD_CMLDS] = pmc_get_transport_fd(p->cmlds.pmc);
+	clock_gettime(CLOCK_MONOTONIC, &now);
+	return port_cmlds_renew(p, now.tv_sec);
+}
+
 void port_disable(struct port *p)
 {
 	int i;
@@ -1877,6 +1992,12 @@ void port_disable(struct port *p)
 		close(p->fda.fd[FD_FIRST_TIMER + i]);
 	}
 
+	if (p->cmlds.pmc) {
+		pmc_destroy(p->cmlds.pmc);
+		p->fda.fd[FD_CMLDS] = -1;
+		p->cmlds.pmc = NULL;
+	}
+
 	/* Keep rtnl socket to get link status info. */
 	port_clear_fda(p, FD_RTNL);
 	clock_fda_changed(p->clock);
@@ -1952,6 +2073,10 @@ int port_initialize(struct port *p)
 		goto no_tmo;
 	}
 
+	if (port_delay_mechanism(p) == DM_COMMON_P2P && port_cmlds_initialize(p)) {
+		goto no_tmo;
+	}
+
 	/* No need to open rtnl socket on UDS port. */
 	if (!port_is_uds(p)) {
 		/*
@@ -2093,6 +2218,65 @@ int process_announce(struct port *p, struct ptp_message *m)
 	return result;
 }
 
+static int process_cmlds(struct port *p)
+{
+	struct cmlds_info_np *cmlds;
+	struct management_tlv *mgt;
+	struct ptp_message *msg;
+	struct TLV *tlv;
+	int err = 0;
+
+	msg = pmc_recv(p->cmlds.pmc);
+	if (!msg) {
+		pr_err("%s: pmc_recv failed", p->log_name);
+		return -1;
+	}
+	if (msg_type(msg) != MANAGEMENT) {
+		pr_err("%s: pmc_recv bad message", p->log_name);
+		err = -1;
+		goto out;
+	}
+	tlv = (struct TLV *) msg->management.suffix;
+	if (tlv->type != TLV_MANAGEMENT) {
+		pr_err("%s: pmc_recv bad message", p->log_name);
+		err = -1;
+		goto out;
+	}
+	mgt = (struct management_tlv *) msg->management.suffix;
+	if (mgt->length == 2 && mgt->id != MID_NULL_MANAGEMENT) {
+		pr_err("%s: pmc_recv bad length", p->log_name);
+		goto out;
+	}
+
+	switch (mgt->id) {
+	case MID_CMLDS_INFO_NP:
+		if (msg->header.sourcePortIdentity.portNumber != p->cmlds.port) {
+			break;
+		}
+		cmlds = (struct cmlds_info_np *) mgt->data;
+		p->peer_delay = nanoseconds_to_tmv(cmlds->meanLinkDelay >> 16);
+		p->peerMeanPathDelay = cmlds->meanLinkDelay;
+		p->nrate.ratio = 1.0 + (double) cmlds->scaledNeighborRateRatio / POW2_41;
+		p->asCapable = cmlds->as_capable;
+		p->cmlds.timer_count = 0;
+		if (p->state == PS_UNCALIBRATED || p->state == PS_SLAVE) {
+			const tmv_t tx = tmv_zero();
+			clock_peer_delay(p->clock, p->peer_delay, tx, tx, p->nrate.ratio);
+		}
+		break;
+	case MID_SUBSCRIBE_EVENTS_NP:
+		break;
+	default:
+		pr_err("%s: pmc_recv bad mgt id 0x%x", p->log_name, mgt->id);
+		err = -1;
+		break;
+	}
+
+out:
+	msg_put(msg);
+	return err;
+}
+
 static int process_delay_req(struct port *p, struct ptp_message *m)
 {
 	struct ptp_message *msg;
@@ -2104,7 +2288,7 @@ static int process_delay_req(struct port *p, struct ptp_message *m)
 		return 0;
 	}
 
-	if (p->delayMechanism == DM_P2P) {
+	if (p->delayMechanism == DM_P2P || p->delayMechanism == DM_COMMON_P2P) {
 		pr_warning("%s: delay request on P2P port", p->log_name);
 		return 0;
 	}
@@ -2269,6 +2453,9 @@ int process_pdelay_req(struct port *p, struct ptp_message *m)
 		return -1;
 	}
 
+	if (p->delayMechanism == DM_COMMON_P2P) {
+		return 0;
+	}
 	if (p->delayMechanism == DM_E2E) {
 		pr_warning("%s: pdelay_req on E2E port", p->log_name);
 		return 0;
@@ -2307,7 +2494,8 @@ int process_pdelay_req(struct port *p, struct ptp_message *m)
 
 	rsp->hwts.type = p->timestamping;
 
-	rsp->header.tsmt               = PDELAY_RESP | p->transportSpecific;
+	rsp->header.tsmt               = m->header.tsmt & 0xf0;
+	rsp->header.tsmt               |= PDELAY_RESP;
 	rsp->header.ver                = ptp_hdr_ver;
 	rsp->header.messageLength      = sizeof(struct pdelay_resp_msg);
 	rsp->header.domainNumber       = m->header.domainNumber;
@@ -2458,10 +2646,15 @@ calc:
 
 	msg_put(p->peer_delay_req);
 	p->peer_delay_req = NULL;
+
+	port_notify_event(p, NOTIFY_CMLDS);
 }
 
 int process_pdelay_resp(struct port *p, struct ptp_message *m)
 {
+	if (p->delayMechanism == DM_COMMON_P2P) {
+		return 0;
+	}
 	if (p->peer_delay_resp) {
                 if (!p->multiple_pdr_detected) {
                         pr_err("%s: multiple peer responses", p->log_name);
@@ -2599,11 +2792,11 @@ void port_close(struct port *p)
 
 struct foreign_clock *port_compute_best(struct port *p)
 {
-	int (*dscmp)(struct dataset *a, struct dataset *b);
+	int (*clk_dscmp)(struct dataset *a, struct dataset *b);
 	struct foreign_clock *fc;
 	struct ptp_message *tmp;
 
-	dscmp = clock_dscmp(p->clock);
+	clk_dscmp = clock_dscmp(p->clock);
 	p->best = NULL;
 
 	if (p->master_only)
@@ -2623,7 +2816,7 @@ struct foreign_clock *port_compute_best(struct port *p)
 
 		if (!p->best)
 			p->best = fc;
-		else if (dscmp(&fc->dataset, &p->best->dataset) > 0)
+		else if (clk_dscmp(&fc->dataset, &p->best->dataset) > 0)
 			p->best = fc;
 		else
 			fc_clear(fc);
@@ -2648,6 +2841,7 @@ static void port_e2e_transition(struct port *p, enum port_state next)
 	case PS_FAULTY:
 	case PS_DISABLED:
 		port_disable(p);
+		sad_set_last_seqid(clock_config(p->clock), p->spp, -1);
 		break;
 	case PS_LISTENING:
 		port_set_announce_tmo(p);
@@ -2661,6 +2855,7 @@ static void port_e2e_transition(struct port *p, enum port_state next)
 			set_tmo_log(p->fda.fd[FD_MANNO_TIMER], 1, -10); /*~1ms*/
 		}
 		port_set_sync_tx_tmo(p);
+		sad_set_last_seqid(clock_config(p->clock), p->spp, -1);
 		break;
 	case PS_PASSIVE:
 		port_set_announce_tmo(p);
@@ -2668,6 +2863,7 @@ static void port_e2e_transition(struct port *p, enum port_state next)
 	case PS_UNCALIBRATED:
 		flush_last_sync(p);
 		flush_delay_req(p);
+		sad_set_last_seqid(clock_config(p->clock), p->spp, -1);
 		/* fall through */
 	case PS_SLAVE:
 		port_set_announce_tmo(p);
@@ -2692,6 +2888,7 @@ static void port_p2p_transition(struct port *p, enum port_state next)
 	case PS_FAULTY:
 	case PS_DISABLED:
 		port_disable(p);
+		sad_set_last_seqid(clock_config(p->clock), p->spp, -1);
 		break;
 	case PS_LISTENING:
 		port_set_announce_tmo(p);
@@ -2706,6 +2903,7 @@ static void port_p2p_transition(struct port *p, enum port_state next)
 			set_tmo_log(p->fda.fd[FD_MANNO_TIMER], 1, -10); /*~1ms*/
 		}
 		port_set_sync_tx_tmo(p);
+		sad_set_last_seqid(clock_config(p->clock), p->spp, -1);
 		break;
 	case PS_PASSIVE:
 		port_set_announce_tmo(p);
@@ -2713,6 +2911,7 @@ static void port_p2p_transition(struct port *p, enum port_state next)
 	case PS_UNCALIBRATED:
 		flush_last_sync(p);
 		flush_peer_delay(p);
+		sad_set_last_seqid(clock_config(p->clock), p->spp, -1);
 		/* fall through */
 	case PS_SLAVE:
 		port_set_announce_tmo(p);
@@ -2737,10 +2936,14 @@ static void bc_dispatch(struct port *p, enum fsm_event event, int mdiff)
 		return;
 	}
 
-	if (p->delayMechanism == DM_P2P) {
+	switch (p->delayMechanism) {
+	case DM_COMMON_P2P:
+	case DM_P2P:
 		port_p2p_transition(p, p->state);
-	} else {
+		break;
+	default:
 		port_e2e_transition(p, p->state);
+		break;
 	}
 
 	if (p->jbod && p->state == PS_UNCALIBRATED && p->phc_index >= 0 ) {
@@ -2841,7 +3044,7 @@ enum fsm_event port_event(struct port *p, int fd_index)
 static enum fsm_event bc_event(struct port *p, int fd_index)
 {
 	enum fsm_event event = EV_NONE;
-	struct ptp_message *msg;
+	struct ptp_message *msg, *dup = NULL;
 	int cnt, fd = p->fda.fd[fd_index], err;
 
 	switch (fd_index) {
@@ -2888,6 +3091,9 @@ static enum fsm_event bc_event(struct port *p, int fd_index)
 	case FD_DELAY_TIMER:
 		pr_debug("%s: delay timeout", p->log_name);
 		port_set_delay_tmo(p);
+		if (p->delayMechanism == DM_COMMON_P2P) {
+			return port_cmlds_timeout(p);
+		}
 		delay_req_prune(p);
 		p->service_stats.delay_timeout++;
 		if (port_delay_request(p)) {
@@ -2934,6 +3140,10 @@ static enum fsm_event bc_event(struct port *p, int fd_index)
 		p->service_stats.unicast_request_timeout++;
 		return unicast_client_timer(p) ? EV_FAULT_DETECTED : EV_NONE;
 
+	case FD_CMLDS:
+		pr_debug("%s: CMLDS push notification", p->log_name);
+		return process_cmlds(p) ? EV_FAULT_DETECTED : EV_NONE;
+
 	case FD_RTNL:
 		pr_debug("%s: received link status notification", p->log_name);
 		rtnl_link_status(fd, p->name, port_link_status, p);
@@ -2958,6 +3168,13 @@ static enum fsm_event bc_event(struct port *p, int fd_index)
 		msg_put(msg);
 		return EV_FAULT_DETECTED;
 	}
+	if (port_has_security(p)) {
+		dup = msg_duplicate(msg, 0);
+		if (!dup) {
+			msg_put(msg);
+			return EV_NONE;
+		}
+	}
 	err = msg_post_recv(msg, cnt);
 	if (err) {
 		switch (err) {
@@ -2969,11 +3186,17 @@ static enum fsm_event bc_event(struct port *p, int fd_index)
 			break;
 		}
 		msg_put(msg);
+		if (dup) {
+			msg_put(dup);
+		}
 		return EV_NONE;
 	}
 	port_stats_inc_rx(p, msg);
 	if (port_ignore(p, msg)) {
 		msg_put(msg);
+		if (dup) {
+			msg_put(dup);
+		}
 		return EV_NONE;
 	}
 	if (msg_sots_missing(msg) &&
@@ -2981,6 +3204,25 @@ static enum fsm_event bc_event(struct port *p, int fd_index)
 		pr_err("%s: received %s without timestamp",
 		       p->log_name, msg_type_string(msg_type(msg)));
 		msg_put(msg);
+		if (dup) {
+			msg_put(dup);
+		}
+		return EV_NONE;
+	}
+	err = sad_process_auth(clock_config(p->clock), p->spp, msg, dup);
+	if (err) {
+		switch (err) {
+		case -EBADMSG:
+			pr_err("%s: auth: bad message", p->log_name);
+			break;
+		case -EPROTO:
+			pr_debug("%s: auth: ignoring message", p->log_name);
+			break;
+		}
+		msg_put(msg);
+		if (dup) {
+			msg_put(dup);
+		}
 		return EV_NONE;
 	}
 	if (msg_sots_valid(msg)) {
@@ -3032,6 +3274,9 @@ static enum fsm_event bc_event(struct port *p, int fd_index)
 	}
 
 	msg_put(msg);
+	if (dup) {
+		msg_put(dup);
+	}
 	return event;
 }
 
@@ -3063,8 +3308,13 @@ int port_prepare_and_send(struct port *p, struct ptp_message *msg,
 			  enum transport_event event)
 {
 	int cnt;
-
-	if (msg_pre_send(msg)) {
+	if (port_has_security(p)) {
+		cnt = sad_append_auth_tlv(clock_config(p->clock), p->spp,
+					  p->active_key_id, msg);
+	} else {
+		cnt = msg_pre_send(msg);
+	}
+	if (cnt) {
 		return -1;
 	}
 	if (msg_unicast(msg)) {
@@ -3255,6 +3505,9 @@ void port_notify_event(struct port *p, enum notification event)
 	case NOTIFY_PORT_STATE:
 		id = MID_PORT_DATA_SET;
 		break;
+	case NOTIFY_CMLDS:
+		id = MID_CMLDS_INFO_NP;
+		break;
 	default:
 		return;
 	}
@@ -3403,6 +3656,8 @@ struct port *port_open(const char *phc_device,
 		config_get_int(cfg, p->name, "power_profile.2017.totalTimeInaccuracy");
 	p->slave_event_monitor = clock_slave_monitor(clock);
 	p->allowedLostResponses = config_get_int(cfg, p->name, "allowedLostResponses");
+	p->spp = config_get_int(cfg, p->name, "spp");
+	p->active_key_id = config_get_uint(cfg, p->name, "active_key_id");
 
 	if (!port_is_uds(p) && unicast_client_initialize(p)) {
 		goto err_transport;
@@ -3432,6 +3687,19 @@ struct port *port_open(const char *phc_device,
 	if (p->net_sync_monitor && !p->hybrid_e2e) {
 		pr_warning("%s: net_sync_monitor needs hybrid_e2e", p->log_name);
 	}
+	if (sad_readiness_check(p->spp, p->active_key_id, clock_config(p->clock))) {
+		pr_err("%s: security readiness check failed", p->log_name);
+		goto err_uc_service;
+	}
+	if (port_has_security(p) && (p->timestamping == TS_ONESTEP ||
+				     p->timestamping == TS_P2P1STEP)) {
+		pr_err("%s: spp not supported on one-step ports", p->log_name);
+		goto err_uc_service;
+	}
+	if (port_has_security(p) && (config_get_int(cfg, NULL, "ptp_minor_version") < 1)) {
+		pr_err("%s: spp needs at least PTPv2.1", p->log_name);
+		goto err_uc_service;
+	}
 
 	/* Set fault timeouts to a default value */
 	for (i = 0; i < FT_CNT; i++) {
