@@ -176,6 +176,7 @@ static int mgt_post_recv(struct management_tlv *m, uint16_t data_len,
 	struct port_properties_np *ppn;
 	struct port_hwclock_np *phn;
 	struct timePropertiesDS *tp;
+	struct cmlds_info_np *cmlds;
 	struct time_status_np *tsn;
 	struct port_stats_np *psn;
 	int extra_len = 0, i, len;
@@ -481,6 +482,14 @@ static int mgt_post_recv(struct management_tlv *m, uint16_t data_len,
 		NTOHL(pwr->networkTimeInaccuracy);
 		NTOHL(pwr->totalTimeInaccuracy);
 		break;
+	case MID_CMLDS_INFO_NP:
+		if (data_len < sizeof(struct cmlds_info_np))
+			goto bad_length;
+		cmlds = (struct cmlds_info_np *)m->data;
+		net2host64_unaligned(&cmlds->meanLinkDelay);
+		NTOHL(cmlds->scaledNeighborRateRatio);
+		NTOHL(cmlds->as_capable);
+		break;
 	case MID_SAVE_IN_NON_VOLATILE_STORAGE:
 	case MID_RESET_NON_VOLATILE_STORAGE:
 	case MID_INITIALIZE:
@@ -514,6 +523,7 @@ static void mgt_pre_send(struct management_tlv *m, struct tlv_extra *extra)
 	struct subscribe_events_np *sen;
 	struct port_properties_np *ppn;
 	struct port_hwclock_np *phn;
+	struct cmlds_info_np *cmlds;
 	struct timePropertiesDS *tp;
 	struct time_status_np *tsn;
 	struct port_stats_np *psn;
@@ -672,6 +682,12 @@ static void mgt_pre_send(struct management_tlv *m, struct tlv_extra *extra)
 		HTONL(pwr->networkTimeInaccuracy);
 		HTONL(pwr->totalTimeInaccuracy);
 		break;
+	case MID_CMLDS_INFO_NP:
+		cmlds = (struct cmlds_info_np *)m->data;
+		host2net64_unaligned(&cmlds->meanLinkDelay);
+		HTONL(cmlds->scaledNeighborRateRatio);
+		HTONL(cmlds->as_capable);
+		break;
 	}
 }
 
@@ -993,6 +1009,26 @@ static void slave_rx_sync_timing_data_pre_send(struct tlv_extra *extra)
 	}
 }
 
+static int auth_post_recv(struct tlv_extra *extra)
+{
+	struct authentication_tlv *auth = (struct authentication_tlv *) extra->tlv;
+
+	if (auth->length < sizeof(*auth)) {
+		return -EBADMSG;
+	}
+
+	NTOHL(auth->keyID);
+
+	return 0;
+}
+
+static void auth_pre_send(struct tlv_extra *extra)
+{
+	struct authentication_tlv *auth = (struct authentication_tlv *) extra->tlv;
+
+	HTONL(auth->keyID);
+}
+
 static int unicast_message_type_valid(uint8_t message_type)
 {
 	message_type >>= 4;
@@ -1175,7 +1211,10 @@ int tlv_post_recv(struct tlv_extra *extra)
 		break;
 	case TLV_CUMULATIVE_RATE_RATIO:
 	case TLV_PAD:
+		break;
 	case TLV_AUTHENTICATION:
+		result = auth_post_recv(extra);
+		break;
 	default:
 		break;
 	}
@@ -1242,7 +1281,10 @@ void tlv_pre_send(struct TLV *tlv, struct tlv_extra *extra)
 		break;
 	case TLV_CUMULATIVE_RATE_RATIO:
 	case TLV_PAD:
+		break;
 	case TLV_AUTHENTICATION:
+		auth_pre_send(extra);
+		break;
 	default:
 		break;
 	}
