@@ -18,6 +18,7 @@
 #include "interface.h"
 #include "phc.h"
 #include "print.h"
+#include "sad.h"
 #include "ts2phc.h"
 #include "version.h"
 
@@ -35,9 +36,10 @@ static void ts2phc_cleanup(struct ts2phc_private *priv)
 	ts2phc_pps_sink_cleanup(priv);
 	if (priv->src)
 		ts2phc_pps_source_destroy(priv->src);
-	if (priv->cfg)
+	if (priv->cfg) {
+		sad_destroy(priv->cfg);
 		config_destroy(priv->cfg);
-
+	}
 	if (priv->agent)
 		pmc_agent_destroy(priv->agent);
 
@@ -418,9 +420,16 @@ static int ts2phc_pps_source_implicit_tstamp(struct ts2phc_private *priv,
 	 * deduce the timestamp (actually only seconds part, nanoseconds are by
 	 * construction zero) of this edge at the emitter based on the
 	 * emitter's current time.
+	 *
+	 * With an NMEA source assume its messages always follow the pulse, i.e.
+	 * assign the timestamp to the previous pulse instead of nearest pulse.
 	 */
-	if (source_ts.tv_nsec > NS_PER_SEC / 2)
+	if (ts2phc_pps_source_get_type(priv->src) == TS2PHC_PPS_SOURCE_NMEA) {
 		source_ts.tv_sec++;
+	} else {
+		if (source_ts.tv_nsec > NS_PER_SEC / 2)
+			source_ts.tv_sec++;
+	}
 	source_ts.tv_nsec = 0;
 
 	tmv = timespec_to_tmv(source_ts);
@@ -433,9 +442,10 @@ static int ts2phc_pps_source_implicit_tstamp(struct ts2phc_private *priv,
 
 static void ts2phc_synchronize_clocks(struct ts2phc_private *priv, int autocfg)
 {
+	struct timespec source_ts, now;
 	tmv_t source_tmv;
 	struct ts2phc_clock *c;
-	int valid, err;
+	int holdover, valid;
 
 	if (autocfg) {
 		if (!priv->ref_clock) {
@@ -449,9 +459,20 @@ static void ts2phc_synchronize_clocks(struct ts2phc_private *priv, int autocfg)
 			return;
 		}
 	} else {
-		err = ts2phc_pps_source_implicit_tstamp(priv, &source_tmv);
-		if (err < 0)
+		valid = !ts2phc_pps_source_implicit_tstamp(priv, &source_tmv);
+	}
+
+	if (valid) {
+		priv->holdover_start = 0;
+		holdover = 0;
+	} else {
+		clock_gettime(CLOCK_MONOTONIC, &now);
+
+		if (!priv->holdover_start)
+			priv->holdover_start = now.tv_sec;
+		if (now.tv_sec >= priv->holdover_start + priv->holdover_length)
 			return;
+		holdover = 1;
 	}
 
 	LIST_FOREACH(c, &priv->clocks, list) {
@@ -468,6 +489,16 @@ static void ts2phc_synchronize_clocks(struct ts2phc_private *priv, int autocfg)
 			continue;
 		}
 
+		if (holdover) {
+			if (c->servo_state != SERVO_LOCKED_STABLE)
+				continue;
+			source_ts = tmv_to_timespec(ts);
+			if (source_ts.tv_nsec > NS_PER_SEC / 2)
+				source_ts.tv_sec++;
+			source_ts.tv_nsec = 0;
+			source_tmv = timespec_to_tmv(source_ts);
+		}
+
 		offset = tmv_to_nanoseconds(tmv_sub(ts, source_tmv));
 
 		if (c->no_adj) {
@@ -479,8 +510,15 @@ static void ts2phc_synchronize_clocks(struct ts2phc_private *priv, int autocfg)
 		adj = servo_sample(c->servo, offset, tmv_to_nanoseconds(ts),
 				   SAMPLE_WEIGHT, &c->servo_state);
 
-		pr_info("%s offset %10" PRId64 " s%d freq %+7.0f",
-			c->name, offset, c->servo_state, adj);
+		if (holdover && c->servo_state != SERVO_LOCKED_STABLE) {
+			pr_info("%s lost holdover lock (offset %10" PRId64 ")",
+				c->name, offset);
+			continue;
+		}
+
+		pr_info("%s offset %10" PRId64 " s%d freq %+7.0f%s",
+			c->name, offset, c->servo_state, adj,
+			holdover ? " holdover" : "");
 
 		switch (c->servo_state) {
 		case SERVO_UNLOCKED:
@@ -559,7 +597,7 @@ static void usage(char *progname)
 
 int main(int argc, char *argv[])
 {
-	int c, err = 0, have_sink = 0, index, print_level;
+	int c, err = 0, have_sink = 0, index, cmd_line_print_level;
 	char uds_local[MAX_IFNAME_SIZE + 1];
 	enum ts2phc_pps_source_type pps_type;
 	struct ts2phc_private priv = {0};
@@ -612,13 +650,12 @@ int main(int argc, char *argv[])
 			config = optarg;
 			break;
 		case 'l':
-			if (get_arg_val_i(c, optarg, &print_level,
+			if (get_arg_val_i(c, optarg, &cmd_line_print_level,
 					  PRINT_LEVEL_MIN, PRINT_LEVEL_MAX)) {
 				ts2phc_cleanup(&priv);
 				return -1;
 			}
-			config_set_int(cfg, "logging_level", print_level);
-			print_set_level(print_level);
+			config_set_int(cfg, "logging_level", cmd_line_print_level);
 			break;
 		case 'm':
 			config_set_int(cfg, "verbose", 1);
@@ -664,6 +701,12 @@ int main(int argc, char *argv[])
 
 	STAILQ_INIT(&priv.sinks);
 
+	if (sad_create(cfg)) {
+		fprintf(stderr, "failed to get security associations\n");
+		ts2phc_cleanup(&priv);
+		return -1;
+	}
+
 	snprintf(uds_local, sizeof(uds_local), "/var/run/ts2phc.%d",
 		 getpid());
 
@@ -744,8 +787,11 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 
+	priv.holdover_length = config_get_int(cfg, NULL, "ts2phc.holdover");
+	priv.holdover_start = 0;
+
 	while (is_running()) {
-		struct ts2phc_clock *c;
+		struct ts2phc_clock *clk;
 
 		if (autocfg) {
 			/* Collect updates from ptp4l */
@@ -759,8 +805,8 @@ int main(int argc, char *argv[])
 				ts2phc_reconfigure(&priv);
 		}
 
-		LIST_FOREACH(c, &priv.clocks, list)
-			ts2phc_clock_flush_tstamp(c);
+		LIST_FOREACH(clk, &priv.clocks, list)
+			ts2phc_clock_flush_tstamp(clk);
 
 		err = ts2phc_pps_sink_poll(&priv);
 		if (err < 0) {
