@@ -792,3 +792,105 @@ int rate_limited(int interval, time_t *last)
 
 	return 0;
 }
+
+const int B64[] = {62, -1, -1, -1, 63, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61,
+	-1, -1, -1, -1, -1, -1, -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
+	14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1, -1,
+	26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43,
+	44, 45, 46, 47, 48, 49, 50, 51};
+static inline int parseB64(const char c)
+{
+	if (c < '+' || c > 'z')
+		return -1;
+	return B64[c - '+'];
+}
+size_t base64_len(const char *str, size_t len)
+{
+	div_t q;
+	int diff = 0;
+
+	if (len == 0)
+		len = strlen(str);
+	if (len < 2)
+		return 0;
+	q = div(len, 4);
+	if (q.rem == 0) {
+		if (str[len - 1] == '=')
+			diff--;
+		if (str[len - 2] == '=')
+			diff--;
+	} else {
+		diff++;
+		if (q.rem > 2)
+			diff++;
+	}
+	return (q.quot * 3) + diff;
+}
+
+bool base64_decode(const char *in_str, size_t in_len, void *out, size_t *out_len)
+{
+	size_t len, i;
+	uint8_t *cur, d[4];
+
+	if (!in_str || !out_len)
+		return false;
+	if (in_len == 0)
+		in_len = strlen(in_str);
+	if (in_len == 0) {
+		*out_len = 0;
+		return true;
+	}
+	if (!out)
+		return false;
+	len = *out_len;
+	cur = (uint8_t *)out;
+	for (; in_len > 4; in_len -= 4) {
+		// Read 4 characters
+		for (i = 0; i < 4; i++) {
+			int r = parseB64(*in_str++);
+			if (r < 0)
+				return false;
+			d[i] = r;
+		}
+		if (len) {
+			*cur++ = ((d[0] << 2) & 0xfc) | ((d[1] >> 4) & 0x03);
+			len--;
+		}
+		if (len) {
+			*cur++ = ((d[1] << 4) & 0xf0) | ((d[2] >> 2) & 0x0f);
+			len--;
+		}
+		if (len) {
+			*cur++ = ((d[2] << 6) & 0xc0) | (d[3] & 0x3f);
+			len--;
+		}
+	}
+	if (in_len < 2)
+		return false;
+	if (in_len == 4 && in_str[3] == '=')
+		in_len--;
+	if (in_len == 3 && in_str[2] == '=')
+		in_len--;
+	for (i = 0; i < in_len; i++) {
+		int r = parseB64(*in_str++);
+		if (r < 0)
+			return false;
+		d[i] = r;
+	}
+	if (len) {
+		*cur++ = ((d[0] << 2) & 0xfc) | ((d[1] >> 4) & 0x03);
+		len--;
+	}
+	if (in_len > 2) {
+		if (len) {
+			*cur++ = ((d[1] << 4) & 0xf0) | ((d[2] >> 2) & 0x0f);
+			len--;
+		}
+		if (in_len == 4 && len) {
+			*cur++ = ((d[2] << 6) & 0xc0) | (d[3] & 0x3f);
+			len--;
+		}
+	}
+	*out_len = *out_len - len;
+	return true;
+}
